@@ -31,15 +31,9 @@
  * SUCH DAMAGE.
  */
 
-#ifdef WIN32
-#include "config-win32.h"
-#else
-#include "config.h"
-#endif
-
 #include "syshead.h"
 
-#ifdef ENABLE_HTTP_PROXY
+#if defined(ENABLE_HTTP_PROXY) || defined(ENABLE_PKCS11) || defined(ENABLE_CLIENT_CR)
 
 #include "base64.h"
 
@@ -86,8 +80,6 @@ base64_encode(const void *data, int size, char **str)
     return strlen(s);
 }
 
-#if NTLM
-
 static int 
 pos(char c)
 {
@@ -143,8 +135,6 @@ base64_decode(const char *str, void *data)
     return q - (unsigned char *) data;
 }
 
-#endif /* NTLM */
-
 #else
 static void dummy(void) {}
-#endif
+#endif /* ENABLE_HTTP_PROXY, ENABLE_PKCS11, ENABLE_CLIENT_CR */
